/*
 *  Copyright (C) 2008, Pedro Carlos da Silva Lara
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.   
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package ec.gf;

import java.math.BigInteger;

public class GaloisField {
	
	/**
	 *  
	 * @param m Grau do corpo de Galois GF(2<sup>m</sup>)
	 * @param primitive Polinmio primitivo de GF(2<sup>m</sup>)
	 */
	
	public GaloisField( int m, BigInteger primitive ) {
		this.m = m;
		this.primitive = primitive;
	}
	
	/**
	 *  
	 * @param m Grau do Corpo de Galois GF(2<sup>m</sup>)
	 * @param primitive Polinmio primitivo de GF(2<sup>m</sup>)
	 */
	
	public GaloisField( int m, String primitive ) {
		this.m = m;
		this.primitive = new BigInteger( primitive );
	}
	/**
	 * Constri um objeto do tipo GaloisField onde o polinmio 
	 * primitivo  da forma: <br>
	 * X<sup>m</sup> + X<sup>k1</sup> + X<sup>k2</sup> + X<sup>k3</sup> + 1
	 * 
	 */
	
	public GaloisField(int m, int k1, int k2, int k3) {
		this.m = m;
		BigInteger TWO = new BigInteger("2");
		
		this.primitive = TWO.pow( m ).
						 add( TWO.pow(k1) ).
						 add( TWO.pow(k2) ).
						 add( TWO.pow(k3) ).
						 add( BigInteger.ONE );
	}
	/**
	 * 
	 * @return O grau do polinmio primitivo
	 */
	public int getDegree() {
		return m;
	}
	/**
	 * 
	 * @return O polinmio primitivo
	 */
	public BigInteger getPrimitive() {
		return primitive;
	}
	/**
	 * Compara dois objetos do tipo <b>GaloisField</b>
	 * 
	 * @param A Objeto que ser comparado 
	 * @return true se so iguais, false caso contrrio
	 */
	public boolean equals(GaloisField A) {
		return (A.getPrimitive().equals( primitive ) &
				A.getDegree() == m );
	}

	private BigInteger primitive;
	private int m;
}
