/*
 *  Copyright (C) 2008, Pedro Carlos da Silva Lara
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.   
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package ec.gf;


import ec.NAF;
import java.math.BigInteger;
import java.util.HashMap;

import ec.zp.Utils;

public class Point {
	
	/**
	 * Cria um ponto no infinito
	 * 
	 */
	
	public Point(Curve E) {
		this.setCurve(E);
		setInfty( true );
		GaloisField GF = new GaloisField(E.getA().getGF().getDegree(), E.getA().getGF().getPrimitive() );
		setX(new ElementGF( "0", GF ));
		setY(new ElementGF( "0", GF ));
	}
	
	public Point(ElementGF x, ElementGF y, Curve E ) {
		setInfty(false);
		this.setX(x);
		this.setY(y);
		this.setCurve(E);
	}
		
	
	public void setInfty(boolean infty) {
		this.infty = infty;
	}

	public boolean isInfty() {
		return infty;
	}

	public void setX(ElementGF x) {
		this.x = x;
	}

	public ElementGF getX() {
		return x;
	}

	public void setY(ElementGF y) {
		this.y = y;
	}

	public ElementGF getY() {
		return y;
	}

	public void setCurve(Curve curve) {
		E = curve;
	}

	public Curve getCurve() {
		return E;
	}
	
	public boolean getInfty() {
		return infty;
	}
	
	public String getString() {
		if( infty == true ) {
			return new String( "infty" );
		} else {
			return new String( "("+x.get().toString()+", "+y.get().toString()+")");
		}
	}
	
	public Point negate() {
		return new Point( x, x.add( y ), E );
	}
	
	public Point add( Point R ) {
		
		if( R.getInfty() ) 
			return this;
		
		if( infty == true )
			return R;
		
		if( x.add( R.getX() ).get().equals( BigInteger.ZERO ) )
			return new Point( E );
		
		if( this.equals( R ) )
			return doublePoint();		
		
		ElementGF t   = (y.add( R.getY() )).divide( x.add( R.getX() ) );
		ElementGF x_r = t.square().add( t ).add( x ).add( R.getX() ).add( E.getA() );
		ElementGF y_r = (x.add( x_r )).multiply( t ).add( x_r ).add( y );
		
		return new Point( x_r, y_r, E );
	}
	
	public Point multiply( BigInteger K ) {
        Point Q   = new Point( E );
        Point P   = this;
        
        int numBitsOfK = K.bitLength();
        
        for ( int i = numBitsOfK - 1; i >= 0; i-- ) {
           	Q = Q.doublePoint();
           	if( K.testBit( i ) )
                Q = Q.add( P );
        }
        
        return Q;
    }
	
	public Point multiply_NAF2w( BigInteger K, byte w ) {
    	HashMap<Integer, Point> h = NAF.PreComputingNAF2w( this, w );
        
        Point Q   = new Point( E );
        int [] k = Utils.NAF( K, w );
        
        for ( int  i = k.length - 1 ; i >= 0; i-- ) {
        	Q = Q.doublePoint();
        	if( k[i] != 0 ) {
        		if( k[i] > 0 ) {
        			Q = Q.add( h.get( k[i] ) );
        		} else {
        			Q = Q.add( h.get( -k[i] ).negate() );
        		}
        	}
        }
        return Q;
    }
	
	public Point doublePoint() {
	
		if( x.get().equals( BigInteger.ZERO ) )
			return new Point( E );
		
		
		ElementGF t = x.add( y.divide( x ) );
		ElementGF x_r = t.square().add( t ).add( E.getA() );
        ElementGF y_r = x.square().add( t.add( new ElementGF( "1", x.getGF() )).multiply(x_r) );
        
        return new Point( x_r, y_r, E );
	}
	
	private boolean infty;
	private ElementGF x;
	private ElementGF y;
	private Curve E;
}
