/*
 *  Copyright (C) 2008, Pedro Carlos da Silva Lara
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version. 
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.   
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package ec.zp;

import java.math.BigInteger;

/**
 * Esta classe encapsula os parametros usuais de uma
 * curva eliptica sobre Zp
 * 
 * @author Pedro Carlos da Silva Lara
 */
public class Curve {
    
	/**
     * Constri uma curva sobre Zp 
     * 
     * @param a parametro da curva <b>y^2 = x^3 a*x + b</b>
     * @param b parametro da curva <b>y^2 = x^3 a*x + b</b>
     * @param p numero primo que define o corpo finito Zp
     * 
     */
    public Curve( BigInteger a, BigInteger b, BigInteger p ) {
        this.a = a;
        this.b = b;
        this.p = p;
    }
    
    /**
     * Construtor por cpia     
     *  
     * @param E curva que ser copiada 
     */
    
    public Curve( Curve E ) {
        a = E.getA();
        b = E.getB();
        p = E.getP();
    }
    /**
     * @return O primo que define Zp 
     */
    public BigInteger getP() {
        return p;
    }
    
    /**
     * 
     * @return parametro a da curva <b>y^2 = x^3 a*x + b</b>
     */
    
    public BigInteger getA() {
        return a;
    }
    
    /**
     * 
     * @return parametro <b>b</b> da curva <b>y^2 = x^3 a*x + b</b>
     */
    
    public BigInteger getB() {
        return b;
    }
    
    public String getString() {
        return "(" + a.toString() + ", " + b.toString() + ", " + p.toString() + ")";
    }
    
    /**
     * Compara a curva <b>this</b> com a curva <b>E</b>
     *   
     * @param E curva que ser comparada
     * @return true se so iguais, false caso contrrio
     */
    
    public boolean equals( Curve E ) {
        return ( this.a.equals( E.getA() ) && 
                 this.b.equals( E.getB() ) &&
                 this.p.equals( E.getP() ) );
    }
    
    private BigInteger a;
    private BigInteger b;
    private BigInteger p;
}
